@@ -1,4 +1,4 @@
-/* $Id: gvplugin_loadimage.h,v 1.13 2011/01/25 16:30:49 ellson Exp $ $Revision: 1.13 $ */
+/* $Id$ $Revision$ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /*************************************************************************
@@ -27,7 +27,7 @@ extern "C" {
 #endif
 
 /*visual studio*/
-#ifdef WIN32_DLL
+#ifdef _WIN32
 #ifndef GVC_EXPORTS
 #define extern __declspec(dllimport)
 #endif
